
#ifndef _OSLEO_H_
#define _OSLEO_H_

#ifdef _LANGUAGE_C_PLUS_PLUS
extern "C" {
#endif

#include <ultra64.h>

#if defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS)

#define	OS_PRIORITY_LEOMGR	OS_PRIORITY_PIMGR

/*
 * Definition for osLeoSpdlMotor()
 */
#define ACTIVE		0
#define	STANDBY		1
#define	SLEEP		2

#define	NUM_LBAS		4292 	/* total number of LBAs */

/*
 * Error codes
 */
#define LEO_ERROR_NO_SEEK_COMPLETE                 0x02
#define LEO_ERROR_WRITE_FAULT                      0x03
#define LEO_ERROR_DRIVE_NOT_READY                  0x04
#define LEO_ERROR_NO_REFERENCE_POSITION_FOUND      0x06
#define LEO_ERROR_DEVICE_COMMUNICATION_FAILURE     0x08
#define LEO_ERROR_TRACK_FOLLOWING_ERROR            0x09
#define LEO_ERROR_TRACKING_OR_SPDL_SERVO_FAILURE   0x09
#define LEO_ERROR_UNRECOVERED_READ_ERROR           0x11
#define LEO_ERROR_INVALID_COMMAND_OPERATION_CODE   0x20
#define LEO_ERROR_LBA_OUT_OF_RANGE                 0x21
#define LEO_ERROR_INVALID_FIELD_IN_COMMAND_BLOCK   0x24
#define LEO_ERROR_WRITE_PROTECT_ERROR              0x27
#define LEO_ERROR_MEDIUM_MAY_HAVE_CHANGED          0x28
#define LEO_ERROR_POWERONRESET_DEVICERESET_OCCURED 0x29
#define LEO_ERROR_OVERRUN_ERROR                    0x2c
#define LEO_ERROR_COMMAND_CLEARED_BY_HOST          0x2f
#define LEO_ERROR_INCOMPATIBLE_MEDIUM_INSTALLED    0x30
#define LEO_ERROR_UNKNOWN_FORMAT                   0x30
#define LEO_ERROR_MEDIUM_NOT_PRESENT               0x3a
#define LEO_ERROR_DIAGNOSTIC_FAILURE               0x40
#define LEO_ERROR_COMMAND_PHASE_ERROR              0x4a
#define LEO_ERROR_DATA_PHASE_ERROR                 0x4b
#define LEO_ERROR_ABORT              		   0x80
#define LEO_ERROR_INVALID_COMMAND    		   0x81
#define LEO_ERROR_NO_DEVICE         		   0x83

typedef struct
{
  LEOCmdHeader		header;
  union
  {
    struct
    {
      u32           lba;
      u32           xfer_blks;
      u64          *buff_ptr;
      u32           rw_bytes;
      OSMesgQueue  *post;
    } readwrite;
    struct
    {
      u32           lba;
      OSMesgQueue  *post;
    } seek;
    struct
    {
      void         *buffer_pointer;
      OSMesgQueue  *post;
    } readdiskid;
    OSMesgQueue	*post;
  } data;

} LEOCmd;

#define	_nbytes		readwrite.rw_bytes
#define	_result		header.status

#define GET_ERROR(x) ((x._result == LEO_STATUS_GOOD) ? 0 : \
((x._result == LEO_STATUS_CHECK_CONDITION) ? x.header.sense : x._result))

typedef struct
{
  u8    drive;
  u8    driver;
} LEOVersion;

typedef	struct
{
  u32	startLBA;
  u32	endLBA;
  u32	nbytes;
} LEOCapacity;

typedef struct
{
  u16	pad;
  u8	year;
  u8	month;
  u8	day;
  u8	hour;
  u8	minute;
  u8	second;
} LEODiskTime;

typedef	struct
{
  u64		lineNumber;
  LEODiskTime	time;
} LEOSerialNum;

typedef struct
{
  u8		gameName[4];
  u8		gameVersion;
  u8		diskNumber;
  u16		diskUsage;
  LEOSerialNum	serialNumber;
  u64		reserve;
} LEODiskID;

typedef	u8	LEOSpdlMode;

typedef u8	LEOStatus;

/*
 * Prototypes
 */

/* Initialize routine */
extern	s32 LeoCreateLeoManager(OSPri, OSPri);

/* Synchronous functions */
extern	s32 LeoClearQueue(void);
extern 	s32 LeoByteToLBA(s32, u32, s32 *);
extern	s32 LeoLBAToByte(s32, u32, s32 *);
extern	s32 LeoReadCapacity(LEOCapacity *, s32);
extern	s32 LeoInquiry(LEOVersion *);
extern	s32 LeoTestUnitReady(LEOStatus *);

/* Asynchronous functions */
extern	s32 LeoSpdlMotor(LEOCmd *, LEOSpdlMode, OSMesgQueue *);
extern	s32 LeoSeek(LEOCmd *, u32, OSMesgQueue *);
extern	s32 LeoRezero(LEOCmd *, OSMesgQueue *);
extern	s32 LeoReadWrite(LEOCmd *, s32, u32, void *,
			 u32, OSMesgQueue *);
extern	s32 LeoReadDiskID(LEOCmd *, LEODiskID *, OSMesgQueue *);


#endif  /* defined(_LANGUAGE_C) || defined(_LANGUAGE_C_PLUS_PLUS) */

#ifdef _LANGUAGE_C_PLUS_PLUS
}
#endif

#endif /* !_OSLEO_H */
